## --------------------------------------------------------------------------
##
## Script name: Iso2k sample code
##
## Purpose of script: Filter records from full Iso2k database
##
## Script author: Dr Georgina Falster
##
## Date created: 2019-12-30
##
## Email: gfalster@wustl.edu
##
## Citation: Konecky et al. (2020) The Iso2k Database: A global compilation of paleo-d18O and d2H records to aid understanding of Common Era climate
## Database is available for download from https://doi.org/10.6084/m9.figshare.11553162 (McKay and Konecky, 2020)
## --------------------------------------------------------------------------
##
## Notes: Current for database version 0.14.2
##   
##
## --------------------------------------------------------------------------

# =============================================================================
# set display options
# =============================================================================

options(scipen = 10, digits = 4) # run this if you don't like scientific notation in your outputs

# =============================================================================
# load required packages
# =============================================================================

library("tidyverse")

# =============================================================================
# load Iso2k database
# =============================================================================

load("iso2kv0_14_2.RData")

rm(D, TS) # Remove extraneous objects

# =============================================================================
# look at individual records
# =============================================================================

## first, a couple of ways to search by Iso2k unique identifier, or by site name.

## extract all Iso2k UIDs
TSids <- as.character(sapply(sTS, "[[", "paleoData_iso2kUI"))

## filter chosen record from full record list
whichRecordName <- which(TSids == "MS12CCCH01b")
recordTS <- sTS[whichRecordName]

## extract dataset names
siteNames <- as.character(sapply(sTS, "[[", "geo_siteName"))

## filter datasets containing the desired site name from the full record list
whichSite <- which(grepl("bahamas", siteNames, ignore.case=TRUE))
selectedSiteTS <- sTS[whichSite]

View(selectedSiteTS)

## view site names for the Bahamas records
as.character(sapply(selectedSiteTS, "[[", "geo_siteName")) %>%
  unique() %>%
  sort()

# =============================================================================
# initial filtering of the database, using Level 1 fields
# =============================================================================

## starting with the entire database, filter for records that
  # have water isotope proxy data i.e.  d18O or d2H,
  # have units in per mille, and
  # are flagged as the primary timeseries
  
variableName <- as.character(sapply(sTS, "[[", "paleoData_variableName"))
units <- as.character(sapply(sTS, "[[", "paleoData_units"))
primaryTS <- as.character(sapply(sTS, "[[", "paleoData_iso2kPrimaryTimeseries"))

## create filters for records with isotope data
isd18O <- which(variableName == "d18O" & primaryTS == "TRUE"  & units == "permil")
isd2H <- which(variableName == "d2H" & primaryTS == "TRUE"  & units == "permil")
isIso <- c(isd18O, isd2H)

allIsoTS <- sTS[isIso] # filter the full timeseries for all records that have O or H isotopes as the variable and are also flagged as primary.

length(allIsoTS) # See how many records are in this filtered subset of the database

# =============================================================================
# additional filtering of the database, using Level 1 or Level 2 fields
# =============================================================================

## from the filtered isotope records, restrict to records where
# the rank 1 interpretation is the isotopic composition of precipitation
# the archive is terrestrial, or
# the inferred material is groundwater or soil water or leaf water

interpretation <- as.character(sapply(allIsoTS, "[[", "isotopeInterpretation1_variableGroup"))
description <- as.character(sapply(allIsoTS, "[[", "paleoData_description"))
inferredMaterial <- as.character(sapply(allIsoTS, "[[", "paleoData_inferredMaterial"))

## check names
unique(description)
unique(inferredMaterial)

## create filters
isPrecipIsotope <- which(interpretation == "P_isotope")
isTerrestrial <- which(inferredMaterial == "lake water" | inferredMaterial == "lagoon water" | 
                         inferredMaterial == "groundwater" | inferredMaterial == "soil water")

isoPIso <- allIsoTS[isPrecipIsotope]
isoTerrestrial <- allIsoTS[isTerrestrial]

## view descriptions for the terrestrial records. How many are there of each?
descriptionTerrestrial <- as.character(sapply(isoTerrestrial, "[[", "paleoData_description"))

descStats <- table(descriptionTerrestrial) %>%
  as.data.frame() %>%
  mutate(perCent = (Freq/length(isoTerrestrial))*100)

descStats

# =============================================================================
# additional filtering of the database, using Level 2 fields
# =============================================================================

## from the filtered isotope records, restrict to records where
# the authors' primary climatic interpretation was based on the amount effect

climInterpretation <- as.character(sapply(allIsoTS, "[[", "climateInterpretation1_variable"))

## check names
unique(climInterpretation)

## create filter
isAmount <- which(grepl("P_", climInterpretation, ignore.case = TRUE) | 
                    grepl("Precipitation", climInterpretation, ignore.case = TRUE) | climInterpretation == "P")

isoAmount <- allIsoTS[isAmount]
 
## how many records were retained? 
length(isoAmount)

## what are the names of the climatic interpretations that were retained?
isoAmountNames <- as.character(sapply(isoAmount, "[[", "climateInterpretation1_variable")) %>%
  unique() %>%
  sort()

isoAmountNames # maybe some additional filtering to be done here

# =============================================================================
# extract summary information and store in a data frame for easy viewing
# =============================================================================

isotopeDataSummary <- matrix(NA, length(allIsoTS), 9) # Pre-assign storage and give appropriate headers.
colnames(isotopeDataSummary) <- c("archive", "elevation", "siteName", "latitude", "longitude", "uniqueID", "inferredMaterial", "inferredMaterialGroup", "variableName")

## extract information from `allIsoTS`, and store it in `isotopeDataSummary`. 
## note that where a dataset is missing an entry for a field, it is instead filled in with an `NA`. 

for(i in 1:length(allIsoTS)){               
  isotopeDataSummary[i,1] <- allIsoTS[[i]]$archiveType
  isotopeDataSummary[i,2] <- if (is.null(allIsoTS[[i]]$geo_elevation)) {
    NA
  } else {
    allIsoTS[[i]]$geo_elevation
  }
  isotopeDataSummary[i,3] <- allIsoTS[[i]]$geo_siteName
  isotopeDataSummary[i,4] <- allIsoTS[[i]]$geo_latitude
  isotopeDataSummary[i,5] <- allIsoTS[[i]]$geo_longitude
  isotopeDataSummary[i,6] <- allIsoTS[[i]]$paleoData_TSid
  isotopeDataSummary[i,7] <- if 
  (is.null(allIsoTS[[i]]$paleoData_inferredMaterial)) {
    NA
  } else {
    allIsoTS[[i]]$paleoData_inferredMaterial
  }
  isotopeDataSummary[i,8] <- if 
  (is.null(allIsoTS[[i]]$paleoData_inferredMaterialGroup)) {
    NA
  } else {
    allIsoTS[[i]]$paleoData_inferredMaterialGroup
  }
  isotopeDataSummary[i,9] <- allIsoTS[[i]]$paleoData_variableName
}

isotopeDataSummary <- as.data.frame(isotopeDataSummary)

# # make sure anything that should be a number, is a number
isotopeDataSummary$elevation <- as.numeric(as.character(isotopeDataSummary$elevation)) 
isotopeDataSummary$latitude <- as.numeric(as.character(isotopeDataSummary$latitude)) 
isotopeDataSummary$longitude <- as.numeric(as.character(isotopeDataSummary$longitude)) 

View(isotopeDataSummary)
